"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const set_timeout_async_1 = require("./set-timeout-async");
async function renderingTimeout(timeout) {
    await set_timeout_async_1.setTimeoutAsync(timeout);
    throw Error(`Got rendering timeout after ${timeout} ms.`);
}
class AsyncSsrManager {
    constructor(timeout) {
        this.timeout = timeout;
        this.rerenderPromises = new Set();
    }
    async renderUntilCompleted(render) {
        const renderPromise = this.renderingLoop(render);
        if (typeof this.timeout !== 'number') {
            console.warn('No timeout is configured for the Async SSR Manager. This could lead to unexpectedly long render times or, in the worst case, never resolving render calls!');
            return renderPromise;
        }
        return Promise.race([renderPromise, renderingTimeout(this.timeout)]);
    }
    rerenderAfter(promise) {
        this.rerenderPromises.add(promise);
    }
    async renderingLoop(render) {
        let html = render();
        // During a render pass, rerender promises might be added via the
        // rerenderAfter method.
        while (this.rerenderPromises.size > 0) {
            await Promise.all(this.rerenderPromises.values());
            this.rerenderPromises.clear();
            html = render();
        }
        return html;
    }
}
exports.AsyncSsrManager = AsyncSsrManager;
//# sourceMappingURL=async-ssr-manager.js.map